#include <iostream>   // cout
#include <iomanip> 
#include <vector> 
#include "strategy.h" 


using namespace std; 

#define N 6 // number of strategies 
#define TRIALS 100 
#define PRINT_TRIALS 30  

int rewards[2][2] = { { CC, CD }, { DC, DD } }; 


class result {
public:
  int first_player;
  int second_player; 
};  

result compete(Strategy& strat1, Strategy& strat2) {

  int res1, res2;
  int rec1[TRIALS];
  int rec2[TRIALS]; 

  strat1.reset(); strat2.reset(); // prepare for competition 

  result res; 
  res.first_player = 0;
  res.second_player = 0; 

  // compete 
  for (int i=0; i<TRIALS; i++) {
   
    // ask for decision 
    rec1[i] = res1 = strat1(); 
    rec2[i] = res2 = strat2();
    
    // rewards
    strat1.reward( rewards[res1][res2] ); 
    strat2.reward( rewards[res2][res1] ); 
  
    res.first_player += rewards[res1][res2]; 
    res.second_player += rewards[res2][res1]; 
  }

  cout << strat1.getName() << " vs. " << strat2.getName() << endl; 
  cout <<"Player1:  ";
  for (int i=0; i<PRINT_TRIALS; i++) cout << ((rec1[i]) ? "D" : "C");
  cout << " => " << res.first_player << endl; 
  cout << "Player2:  "; 
  for (int i=0; i<PRINT_TRIALS; i++) cout << ((rec2[i]) ? "D" : "C"); 
  cout << " => " << res.second_player << endl << endl; 

  
  return res; 
} 


int main() {

  // array of strategies 
  vector<Strategy*> strats; 
  strats.resize(N);

  strats[0] = new RandomStrategy(); 
  strats[1] = new AlwaysCooperate(); 
  strats[2] = new AlwaysDeceive();
  strats[3] = new CopyPartner();
  strats[4] = new Alternate(); 
  strats[5] = new Pavlov(); 

  
  int results[N][N]; 
  int sum_results[N]; 
  for (int i=0; i<N; i++) sum_results[i] = 0; 

  result res; 

  // compete 
  for (int i=0; i< N; i++)  
    for (int j=0; j<=i; j++) {
      res = compete( *strats[i], *strats[j] ); 
      results[i][j] = res.first_player;
      if (i!=j) 
	results[j][i] = res.second_player;
      else { // i == j 
	results[j][i] += res.second_player;
	results[j][i] /= 2; 
      }
      sum_results[i] += results[i][j]; 
      sum_results[j] += results[j][i]; 
    } 
  
  /*
  // print results
  for (int i=0; i<N; i++) {
    for (int j=0; j<N; j++) {
      cout <<  results[i][j] << " "; 
    } 
    cout << endl; 
  } 
  */

  for (int i=0; i<N; i++) 
    cout << setw(30) << strats[i]->getName() << ":  " << sum_results[i]/6 << endl; 
  
  
  

  for (unsigned i=0; i<strats.size(); i++) 
    delete strats[i]; 

  return 0; 
} 
